/**********************************************************************************
 * Control Program to exercise the ADA2200 Lockin Amplifier
 *   Uses Teensy 3.1 to contol and process data.
 * 
 *  2/18/18                   -jkl
 */
// LockIN Amplifier testing using the ADC
// FreqTimer2 is used for the clock signal to ADA2200.
// http://www.arduino.cc/playground/Code/FrequencyTimer2

#include <SPI.h>
#include <FrequencyTimer2.h>
#include "ADC.h"

//#define PinInt1 20
const int PinInt1 = 20;     // Interrupt from ADA2200. Time to Sample!
//const int ledPin = LED_BUILTIN;
const int MAX_SAMPLES = 1000;
const int ARR_LEN = 8;

const int readPin = A3;

//const int readPeriod = 5000; // us
// SPI chip select
const int slaveAPin = 10;   // SPI Address pin

// set up the speed, mode and endianness of SPI device
SPISettings settingsA(2000000, MSBFIRST, SPI_MODE3); 

ADC *adc = new ADC(); // adc object

  uint8_t val;    // used for SPI
void setup() {
  pinMode(FREQUENCYTIMER2_PIN, OUTPUT);    // This is pin digital 5 in arduino land or pin 7 as native pin number.
  
  pinMode(PinInt1, INPUT); // sets the digital pin as input
  attachInterrupt(PinInt1, isrService, FALLING); // interupt 1 

//    pinMode(ledPin, OUTPUT); // led blinks every loop
    pinMode (slaveAPin, OUTPUT);
    digitalWrite (slaveAPin, HIGH);

    pinMode(readPin, INPUT);
      // initialize SPI:
    SPI.setSCK(14);    // use alternate SPI clock pin
    SPI.begin(); 
    
    Serial.begin(9600);

    delay(1000);
    ///// ADC0 ////
    // reference can be ADC_REF_3V3, ADC_REF_1V2 (not for Teensy LC) or ADC_REF_EXT.
    //adc->setReference(ADC_REF_1V2, ADC_0); // change all 3.3 to 1.2 if you change the reference to 1V2

    adc->setAveraging(0); // set number of averages
    adc->setResolution(12); // set bits of resolution

    // it can be ADC_VERY_LOW_SPEED, ADC_LOW_SPEED, ADC_MED_SPEED, ADC_HIGH_SPEED_16BITS, ADC_HIGH_SPEED or ADC_VERY_HIGH_SPEED
    // see the documentation for more information
    adc->setConversionSpeed(ADC_CONVERSION_SPEED::LOW_SPEED); // change the conversion speed
    // it can be ADC_VERY_LOW_SPEED, ADC_LOW_SPEED, ADC_MED_SPEED, ADC_HIGH_SPEED or ADC_VERY_HIGH_SPEED
    adc->setSamplingSpeed(ADC_SAMPLING_SPEED::LOW_SPEED); // change the sampling speed

    adc->enableInterrupts(ADC_0);
    
  Serial.println("You can issue commands like:");
  Serial.println("  123g - gather 123 samples (max 1000)");
  Serial.println();
  FrequencyTimer2::setPeriod(100);  // 10KHz is minimum ADA2200 clock rate
  FrequencyTimer2::enable();
}

// variables shared between interrupt context and main program
// context must be declared "volatile".
volatile uint16_t count = 0;
uint16_t vals[MAX_SAMPLES];
int16_t arr[ARR_LEN] = { 2,4,6,8,1,3,5,7};
volatile int16_t target = 0;
int16_t starget = 0;  // saved for data processing
int8_t i_q = 0;      // flag: 0=>i, 1=>q

void loop() {
  static int16_t v = 0;
  int sum = 0;
  while ( Serial.available()) {
    char ch = Serial.read();
    switch(ch) {
      case '0'...'9':
        v = v * 10 + ch - '0';
        break;
      case 'g':
        // set the adc target to the input and set the count to zero to start collecting
        // send to Processing when done.
        // !!!!! could require stopping interrupts !!!!!
        if (v >= MAX_SAMPLES){
          Serial.print("Max is ");
          Serial.println(MAX_SAMPLES, DEC);
        } else {
          target = v;
          starget = v;
          count = 0;      // starts gathering here
          Serial.print("set target = ");
          Serial.println(target, DEC);
        }
        v = 0;
        break;
      case 'i':
        // select the in phase mode
        Serial.println("In Phase");
        i_q = 0;    // set flag
        SPI.beginTransaction(settingsA);
        digitalWrite (slaveAPin, LOW);
        // set up 0000
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x00);        // SDO disabled
        digitalWrite (slaveAPin, HIGH);
        digitalWrite (slaveAPin, LOW);
        // set up 0028
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x28);
        //val = SPI.transfer(0x01);        // Enable CLKIN
        val = SPI.transfer(0x03);        // Enable CLKIN & Positive in only
        digitalWrite (slaveAPin, HIGH);
        digitalWrite (slaveAPin, LOW);
        // set up 0029
//        val = SPI.transfer(0x00);
//        val = SPI.transfer(0x29);
//        //val = SPI.transfer(0x01);        // SPARE
//        val = SPI.transfer(0x24);        // SYNCO Enable Edge 1 to 16
//        digitalWrite (slaveAPin, HIGH);
//        digitalWrite (slaveAPin, LOW);
        // set up 002a
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x21);
        val = SPI.transfer(0x18);        // Enable RCLK out
        //val = SPI.transfer(0x58);        // Enable RCLK out and Q result
        //val = SPI.transfer(0x1d);        // Enable RCLK out, 1.2Vocm
        digitalWrite (slaveAPin, HIGH);
        digitalWrite (slaveAPin, LOW);
        // set up 002b
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x2b);
        val = SPI.transfer(0x02);        // RCLK is divide by 8 shud be default?
        digitalWrite (slaveAPin, HIGH);
        SPI.endTransaction();
        break;
      case 'q':
        // select the quadrature mode
        Serial.println("Quadrature");
        i_q = 1;    // set flag
        SPI.beginTransaction(settingsA);
        digitalWrite (slaveAPin, LOW);
        // set up 0000
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x00);        // SDO disabled
        digitalWrite (slaveAPin, HIGH);
        digitalWrite (slaveAPin, LOW);
        // set up 0028
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x28);
        //val = SPI.transfer(0x01);        // Enable CLKIN
        val = SPI.transfer(0x03);        // Enable CLKIN & Positive in only
        digitalWrite (slaveAPin, HIGH);
        digitalWrite (slaveAPin, LOW);
        // set up 002a
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x2a);
        //val = SPI.transfer(0x18);        // Enable RCLK out
        val = SPI.transfer(0x58);        // Enable RCLK out and Q result
        //val = SPI.transfer(0x1d);        // Enable RCLK out, 1.2Vocm
        digitalWrite (slaveAPin, HIGH);
        digitalWrite (slaveAPin, LOW);
        // set up 002b
        val = SPI.transfer(0x00);
        val = SPI.transfer(0x2b);
        val = SPI.transfer(0x02);        // RCLK is divide by 8 shud be default?
        digitalWrite (slaveAPin, HIGH);
        SPI.endTransaction();
        break;
      case 'r':
        //Serial.println("Nothing happened");
        Serial.println("Resetting");
        // core reset
        SPI.beginTransaction(settingsA);
        digitalWrite (slaveAPin, LOW);
        // set up reset register
        val = SPI.transfer(0x00);
        //val = SPI.transfer(0x2d);
        val = SPI.transfer(0x00);
        //val = SPI.transfer(0x01);        // Assert core reset  Can't make this work??
        val = SPI.transfer(0x80);        // Assert reset - can use 0x81 or 0x01 also
        digitalWrite (slaveAPin, HIGH);  // reset via register 0x0000 should not need another write
        SPI.endTransaction();
        break;
      case 'a':
        Serial.println("test averaging");
        int aCount = starget;
        int reps = 0;
        while (aCount >= ARR_LEN){
         reps++;
         aCount -= ARR_LEN;
        }
        //Serial.print(starget,DEC);
        //Serial.print(" int div 8 ");
        //Serial.print(" is ");
        //Serial.println(reps);
        i_q == 0? Serial.print("I"): Serial.print
        ("Q");
        Serial.println(" components");
        while (reps >0){
          for (int i=0; i < ARR_LEN; i++) {
            arr[i] = vals[((reps -1) * ARR_LEN) + i];
          }
          for (int i=0; i < ARR_LEN; i++) {
            Serial.print(arr[i], DEC);
            Serial.print(" ");
          }
          Serial.println();
          for (int i=1; i < ARR_LEN; i++) {    // sort the array for median
            int x = arr[i];
            int j = i;
            while ((j>0) && (arr[j-1] > x)){
              arr[j] = arr[j-1];
              j--;
            }
            arr[j] = x;
          }
          for (int i=0; i < ARR_LEN; i++) {
            Serial.print(arr[i], DEC);
            Serial.print(" ");
            sum = sum+arr[i];
          }
          Serial.println();
          Serial.print("Median: ");
          Serial.println(arr[4]*3.3/adc->getMaxValue());
          Serial.print("Average: ");
          Serial.println((sum / 8)*3.3/adc->getMaxValue());
          sum = 0;
          
          reps--;
        }
          
       break;

    }
  }
   // if ADC count = target, send data to Processing 
   if ((count == target) && (target > 0)){
     i_q == 0? Serial.print("I"): Serial.print("Q");
     Serial.println(" components");
     while (target > 0) {
       Serial.println(vals[count-target]*3.3/adc->getMaxValue());
       target--;
       delay(100);    // give Processing time to swallow - TUNE THIS!!
     }
   }
}

// 
void isrService()
{

  adc->startSingleRead(readPin, ADC_0); 

}

// when the measurement finishes, this will be called
///////////!!!!!!!!!!!!!!///////////// This name is magic!!!!!!!!!!!!! It must be this for adc0!!!!!!!!!!!!!
void adc0_isr() {
  // if ADC count is less than target, save the reading and increment count
  if ( count < target){
   vals[count] = adc->readSingle();
   count++;
  }
}

